#!/usr/local/bin/python
# encoding: utf-8
"""
Documentation for polyglot can be found here: http://pypolyglot.readthedocs.org/en/stable

Translate documents and webpages to various markup languages and document formats (html, epub, mobi ..)

Usage:
    polyglot init
    polyglot [-oc] (pdf|html|epub|mobi) <url> [<destinationFolder> -f <filename> -s <pathToSettingsFile>]
    polyglot kindle <url> [-f <filename> -s <pathToSettingsFile>]

Options:
    init                                                            setup the polyglot settings file for the first time
    pdf                                                             print webpage to pdf
    html                                                            parse and download webpage to a local HTML document
    epub                                                            generate an epub format book from a webpage URL
    kindle                                                          send webpage article straight to kindle

    -h, --help                                                      show this help message
    -v, --version                                                   show version
    -o, --open                                                      open the document after creation
    -c, --clean                                                     add polyglot's clean styling to the output document
    <url>                                                           the url of the article's webpage
    -s <pathToSettingsFile>, --settings <pathToSettingsFile>        path to alternative settings file (optional)
    <destinationFolder>                                             the folder to save the parsed PDF or HTML document to (optional)
    -f <filename>, --filename <filename>                            the name of the file to save, otherwise use webpage title as filename (optional)
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
from subprocess import Popen, PIPE, STDOUT
from docopt import docopt
from fundamentals import tools, times
from polyglot import printpdf
from polyglot import htmlCleaner
# from ..__init__ import *


def main(arguments=None):
    """
    *The main function used when ``cl_utils.py`` is run as a single script from the cl, or when installed as a cl command*
    """
    # setup the command-line util settings
    su = tools(
        arguments=arguments,
        docString=__doc__,
        logLevel="WARNING",
        options_first=False,
        projectName="polyglot"
    )
    arguments, settings, log, dbConn = su.setup()

    # unpack remaining cl arguments using `exec` to setup the variable names
    # automatically
    for arg, val in arguments.iteritems():
        if arg[0] == "-":
            varname = arg.replace("-", "") + "Flag"
        else:
            varname = arg.replace("<", "").replace(">", "")
        if isinstance(val, str) or isinstance(val, unicode):
            exec(varname + " = '%s'" % (val,))
        else:
            exec(varname + " = %s" % (val,))
        if arg == "--dbConn":
            dbConn = val
        log.debug('%s = %s' % (varname, val,))

    ## START LOGGING ##
    startTime = times.get_now_sql_datetime()
    log.info(
        '--- STARTING TO RUN THE cl_utils.py AT %s' %
        (startTime,))

    # for k, v in locals().iteritems():
    #     print k, v

    if not destinationFolder:
        destinationFolder = os.getcwd()
    if not filenameFlag:
        filenameFlag = False
    if not cleanFlag:
        readability = False
    else:
        readability = True

    if init:
        from os.path import expanduser
        home = expanduser("~")
        filepath = home + "/.config/polyglot/polyglot.yaml"
        try:
            cmd = """open %(filepath)s""" % locals()
            p = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
        except:
            pass
        try:
            cmd = """start %(filepath)s""" % locals()
            p = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
        except:
            pass

    if pdf:
        filepath = printpdf.printpdf(
            log=log,
            settings=settings,
            url=url,
            folderpath=destinationFolder,
            title=filenameFlag,
            append=False,
            readability=readability
        ).get()

    if html:

        cleaner = htmlCleaner.htmlCleaner(
            log=log,
            settings=settings,
            url=url,
            outputDirectory=destinationFolder,
            title=filenameFlag,  # SET TO FALSE TO USE WEBPAGE TITLE,
            style=cleanFlag,  # add polyglot's styling to the HTML document
            metadata=True,  # include metadata in generated HTML (e.g. title),
            h1=True  # include title as H1 at the top of the doc
        )
        filepath = cleaner.clean()

    if epub:
        from polyglot import ebook
        epub = ebook(
            log=log,
            settings=settings,
            urlOrPath=url,
            title=filenameFlag,
            bookFormat="epub",
            outputDirectory=destinationFolder
        )
        filepath = epub.get()

    if mobi:
        from polyglot import ebook
        mobi = ebook(
            log=log,
            settings=settings,
            urlOrPath=url,
            title=filenameFlag,
            bookFormat="mobi",
            outputDirectory=destinationFolder,
        )
        filepath = mobi.get()

    if kindle:
        from polyglot import kindle
        sender = kindle(
            log=log,
            settings=settings,
            urlOrPath=url,
            title=filenameFlag
        )
        success = sender.send()

    if openFlag:
        try:
            cmd = """open %(filepath)s""" % locals()
            p = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
        except:
            pass
        try:
            cmd = """start %(filepath)s""" % locals()
            p = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
        except:
            pass

    if "dbConn" in locals() and dbConn:
        dbConn.commit()
        dbConn.close()
    ## FINISH LOGGING ##
    endTime = times.get_now_sql_datetime()
    runningTime = times.calculate_time_difference(startTime, endTime)
    log.info('-- FINISHED ATTEMPT TO RUN THE cl_utils.py AT %s (RUNTIME: %s) --' %
             (endTime, runningTime, ))

    return


if __name__ == '__main__':
    main()



import os
import nose
import shutil
import yaml
from polyglot import htmlCleaner, cl_utils
from polyglot.utKit import utKit

from fundamentals import tools

su = tools(
    arguments={"settingsFile": None},
    docString=__doc__,
    logLevel="DEBUG",
    options_first=False,
    projectName="polyglot"
)
arguments, settings, log, dbConn = su.setup()

# # load settings
# stream = file(
#     "/Users/Dave/.config/polyglot/polyglot.yaml", 'r')
# settings = yaml.load(stream)
# stream.close()

# SETUP AND TEARDOWN FIXTURE FUNCTIONS FOR THE ENTIRE MODULE
moduleDirectory = os.path.dirname(__file__)
utKit = utKit(moduleDirectory)
log, dbConn, pathToInputDir, pathToOutputDir = utKit.setupModule()
utKit.tearDownModule()

# load settings
stream = file(
    pathToInputDir + "/example_settings.yaml", 'r')
settings = yaml.load(stream)
stream.close()

import shutil
try:
    shutil.rmtree(pathToOutputDir)
except:
    pass

# Recursively create missing directories
if not os.path.exists(pathToOutputDir):
    os.makedirs(pathToOutputDir)


# xt-setup-unit-testing-files-and-folders


class test_htmlCleaner():

    def test_htmlCleaner_function(self):

        from polyglot import htmlCleaner
        this = htmlCleaner(
            log=log,
            settings=settings,
            url="http://www.thespacedoctor.co.uk/blog/2016/09/26/mysqlSucker-index.html",
            outputDirectory=pathToOutputDir,
            title=False
        )
        this.clean()

        from polyglot import htmlCleaner
        cleaner = htmlCleaner(
            log=log,
            settings=settings,
            url="http://www.thespacedoctor.co.uk/blog/2016/09/26/mysqlSucker-index.html",
            outputDirectory=pathToOutputDir,
            title="my_clean_doc.html"
        )
        cleaner.clean()

    def test_htmlCleaner_function_exception(self):

        from polyglot import htmlCleaner
        try:
            this = htmlCleaner(
                log=log,
                settings=settings,
                fakeKey="break the code"
            )
            this.get()
            assert False
        except Exception, e:
            assert True
            print str(e)

        # x-print-testpage-for-pessto-marshall-web-object

    # x-class-to-test-named-worker-function



#!/usr/local/bin/python
# encoding: utf-8
"""
*Print webpages to PDF*

:Author:
    David Young

:Date Created:
    September 28, 2015
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
import re
import codecs
from subprocess import Popen, PIPE, STDOUT
from docopt import docopt
from fundamentals import tools, times
from polyglot import authenticate

import codecs
# SET ENCODE ERROR RETURN VALUE


def handler(e):
    return (u' ', e.start + 1)
codecs.register_error('dryx', handler)

###################################################################
# CLASSES                                                         #
###################################################################


class printpdf():

    """
    *PDF printer*

    **Key Arguments:**
        - ``log`` -- logger
        - ``settings`` -- the settings dictionary
        - ``url`` -- the webpage url
        - ``title`` -- title of pdf
        - ``folderpath`` -- path at which to save pdf
        - ``append`` -- append this at the end of the file name (not title)
        - ``readability`` -- clean text with readability

    **Usage:**

        To print a webpage to PDF without any cleaning of the content using the title of the webpage as filename:

        .. code-block:: python 

            from polyglot import printpdf
            pdf = printpdf(
                log=log,
                settings=settings,
                url="https://en.wikipedia.org/wiki/Volkswagen",
                folderpath="/path/to/output",
                readability=False
            ).get()

        To give the PDF an alternative title use:

        .. code-block:: python 

            from polyglot import printpdf
            pdf = printpdf(
                log=log,
                settings=settings,
                url="https://en.wikipedia.org/wiki/Volkswagen",
                folderpath="/path/to/output",
                title="Cars",
                readability=False
            ).get()

        Or to append a string to the end of the filename before *.pdf* extension (useful for indexing or adding date created etc):

        .. code-block:: python 

            from datetime import datetime, date, time
            now = datetime.now()
            now = now.strftime("%Y%m%dt%H%M%S")

            from polyglot import printpdf
            pdf = printpdf(
                log=log,
                settings=settings,
                url="https://en.wikipedia.org/wiki/Volkswagen",
                folderpath="/path/to/output",
                append="_"+now,
                readability=False
            ).get()

        To clean the content using the readability parser and apply some simple styling and pretty fonts:

        .. code-block:: python 

            from polyglot import printpdf
            pdf = printpdf(
                log=log,
                settings=settings,
                url="https://en.wikipedia.org/wiki/Volkswagen",
                folderpath=pathToOutputDir,
                readability=True
            ).get()

    """
    # Initialisation

    def __init__(
            self,
            log,
            settings=False,
            url=False,
            title=False,
            folderpath=False,
            append=False,
            readability=True
    ):
        self.log = log
        log.debug("instansiating a new 'print' object")
        self.settings = settings
        self.url = url
        self.folderpath = folderpath
        self.title = title
        self.append = append
        self.readability = readability
        # xt-self-arg-tmpx

        # INITIAL ACTIONS
        # AUTHENTICATE AGAINST READABILITY WEBAPP PARSER CLIENT
        self.parserClient = authenticate.authenticate(
            log=self.log,
            settings=self.settings
        ).get()

        return None

    def get(self):
        """
        *get the PDF*

        **Return:**
            - ``pdfPath`` -- the path to the generated PDF
        """
        self.log.info('starting the ``get`` method')

        # APPEND TO FILENAME?
        if not self.append:
            self.append = ""

        if not self.readability:
            return self._print_original_webpage()
        else:
            return self._print_parsed_webpage()

        self.log.info('completed the ``get`` method')
        return pdfPath

    def _print_original_webpage(
            self):
        """*print the original webpage*

        **Return:**
            - ``pdfPath`` -- the path to the generated PDF
        """
        self.log.info('starting the ``_print_original_webpage`` method')

        if not self.title:
            from polyglot import htmlCleaner
            cleaner = htmlCleaner(
                log=self.log,
                settings=self.settings,
                url=self.url,
                outputDirectory=self.folderpath,
                title=self.title,  # SET TO FALSE TO USE WEBPAGE TITLE,
                style=True,  # add polyglot's styling to the HTML document
                # include metadata in generated HTML (e.g. title),
                metadata=True,
                h1=True  # include title as H1 at the top of the doc
            )
            htmlFile = cleaner.clean()
            basename = os.path.basename(htmlFile)
            title = basename.replace(".html", "")
            os.remove(htmlFile)
        else:
            title = self.title

        # CONVERT TO PDF WITH ELECTON PDF

        url = self.url
        pdfPath = self.folderpath + "/" + title + self.append + ".pdf"
        electron = self.settings["executables"]["electron path"]
        cmd = """%(electron)s -i "%(url)s" -o "%(pdfPath)s" --printBackground """ % locals()
        p = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
        stdout, stderr = p.communicate()
        self.log.debug('output: %(stdout)s' % locals())
        if len(stderr):
            print stderr

        exists = os.path.exists(pdfPath)
        if not exists:
            print "%(pdfPath)s was not generated for some reason - please investigate" % locals()
            sys.exit(0)

        self.log.info('completed the ``_print_original_webpage`` method')
        return pdfPath

    def _print_parsed_webpage(
            self):
        """*print the parsed/cleaned webpage*

        **Return:**
            - ``pdfPath`` -- the path to the generated PDF
        """
        self.log.info('starting the ``_print_parsed_webpage()`` method')

        from polyglot import htmlCleaner
        cleaner = htmlCleaner(
            log=self.log,
            settings=self.settings,
            url=self.url,
            outputDirectory=self.folderpath,
            title=self.title,  # SET TO FALSE TO USE WEBPAGE TITLE,
            style=True,  # add polyglot's styling to the HTML document
            metadata=True,  # include metadata in generated HTML (e.g. title),
            h1=True  # include title as H1 at the top of the doc
        )
        htmlFile = cleaner.clean()
        pdfPath = htmlFile.replace(".html", self.append + ".pdf")

        # CONVERT TO PDF WITH ELECTON PDF
        electron = self.settings["executables"]["electron path"]
        cmd = """%(electron)s -i "%(htmlFile)s" -o "%(pdfPath)s" """ % locals()
        p = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
        stdout, stderr = p.communicate()
        if len(stderr):
            print stderr
        self.log.debug('output: %(stdout)s' % locals())

        # REMOVE HTML FILE
        os.remove(htmlFile)

        exists = os.path.exists(pdfPath)
        if not exists:
            print "%(pdfPath)s was not generated for some reason - please investigate" % locals()
            sys.exit(0)

        self.log.info('completed the ``_print_parsed_webpage()`` method')
        return pdfPath


if __name__ == '__main__':
    main()



#!/usr/local/bin/python
# encoding: utf-8
"""
*Send documents or webpage articles to a kindle device or app*

:Author:
    David Young

:Date Created:
    October 10, 2016
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
from fundamentals import tools
from .ebook import ebook
import shutil
import getpass
import optparse
import os
import smtplib
import sys
import traceback
from StringIO import StringIO
from email import encoders
from email.MIMEBase import MIMEBase
from email.MIMEMultipart import MIMEMultipart
from email.MIMEText import MIMEText
from email.generator import Generator


class kindle(ebook):
    """
    *Send documents or webpage articles to a kindle device or app*

    **Key Arguments:**
        - ``log`` -- logger
        - ``settings`` -- the settings dictionary
        - ``urlOrPath`` -- the url or path to the content source
        - ``title`` -- the title of the output document. I. False then use the title of the original source. Default *False*
        - ``header`` -- content to add before the article/book content in the resulting ebook. Default *False*
        - ``footer`` -- content to add at the end of the article/book content in the resulting ebook. Default *False*

    **Usage:**

        To send content from a webpage article straight to your kindle device or smart phone app, you will first need to populate the email settings with polyglot's settings file at ``~.config/polyglot/polyglot.yaml``, then use the following code:

        .. code-block:: python 

            from polyglot import kindle
            sender = kindle(
                log=log,
                settings=settings,
                urlOrPath="http://www.thespacedoctor.co.uk/blog/2016/09/26/mysqlSucker-index.html",
                header='<a href="http://www.thespacedoctor.co.uk">thespacedoctor</a>',
                footer='<a href="http://www.thespacedoctor.co.uk">thespacedoctor</a>'
            )
            success = sender.send()

        Success is True or False depending on the success/failure of sending the email to the kindle email address(es).
    """
    # Initialisation

    def __init__(
            self,
            log,
            settings,
            urlOrPath,
            title=False,
            header=False,
            footer=False
    ):
        self.log = log
        log.debug("instansiating a new 'ebook' object")
        self.settings = settings
        self.title = title
        self.header = header
        self.footer = footer
        self.urlOrPath = urlOrPath
        self.outputDirectory = "/tmp"
        self.format = "mobi"

        # Initial Actions

        return None

    def send(
            self):
        """*send the mobi book generated to kindle email address(es)*

        **Return:**
            - ``success`` -- True or False depending on the success/failure of sending the email to the kindle email address(es).
        """
        self.log.info('starting the ``send`` method')

        pathToMobi = self.get()

        # create MIME message

        msg = MIMEMultipart()
        msg['From'] = self.settings["email"]["user_email"]
        msg['To'] = ", ".join(self.settings["email"]["kindle_emails"])
        msg['Subject'] = 'Headjack to Kindle'
        text = 'This email has been automatically sent by headjack'
        msg.attach(MIMEText(text))

        basename = os.path.basename(pathToMobi)
        print "Sending the mobi book `%(pathToMobi)s` to Kindle device(s)" % locals()
        msg.attach(self.get_attachment(pathToMobi))

        # convert MIME message to string
        fp = StringIO()
        gen = Generator(fp, mangle_from_=False)
        gen.flatten(msg)
        msg = fp.getvalue()

        # send email
        try:
            mail_server = smtplib.SMTP_SSL(host=self.settings["email"]["smtp_server"],
                                           port=self.settings["email"]["smtp_port"])
            mail_server.login(self.settings["email"]["smtp_login"], self.settings[
                              "email"]["smtp_password"])
            mail_server.sendmail(self.settings["email"]["user_email"], ", ".join(self.settings[
                                 "email"]["kindle_emails"]), msg)
            mail_server.close()
        except smtplib.SMTPException:
            os.remove(pathToMobi)
            self.log.error(
                'Communication with your SMTP server failed. Maybe wrong connection details? Check exception details and your headjack settings file')
            return False

        # os.remove(pathToMobi)

        self.log.info('completed the ``send`` method')
        return True

    def get_attachment(self, file_path):
        '''Get file as MIMEBase message'''

        try:
            file_ = open(file_path, 'rb')
            attachment = MIMEBase('application', 'octet-stream')
            attachment.set_payload(file_.read())
            file_.close()
            encoders.encode_base64(attachment)

            attachment.add_header('Content-Disposition', 'attachment',
                                  filename=os.path.basename(file_path))
            return attachment
        except IOError:
            traceback.print_exc()
            message = ('The requested file could not be read. Maybe wrong '
                       'permissions?')
            print >> sys.stderr, message
            sys.exit(6)

    # xt-class-method



import os
import nose
import shutil
import yaml

from polyglot.utKit import utKit

# load settings
stream = file("/Users/Dave/.config/polyglot/polyglot.yaml", 'r')
settings = yaml.load(stream)
stream.close()


# SETUP AND TEARDOWN FIXTURE FUNCTIONS FOR THE ENTIRE MODULE
moduleDirectory = os.path.dirname(__file__)
utKit = utKit(moduleDirectory)
log, dbConn, pathToInputDir, pathToOutputDir = utKit.setupModule()
utKit.tearDownModule()

# xnose-class-to-test-main-command-line-function-of-module

# import shutil
try:
    shutil.rmtree(pathToOutputDir)
except:
    pass

# Recursively create missing directories
if not os.path.exists(pathToOutputDir):
    os.makedirs(pathToOutputDir)


class test_printpdf():

    def test_printpdf_function(self):

        from polyglot import printpdf
        pdf = printpdf(
            log=log,
            settings=settings,
            url="https://en.wikipedia.org/wiki/Volkswagen",
            folderpath=pathToOutputDir,
            title=False,
            append=False,
            readability=True
        ).get()

    def test_printpdf_original_function(self):

        from polyglot import printpdf
        pdf = printpdf(
            log=log,
            settings=settings,
            url="https://en.wikipedia.org/wiki/Volkswagen",
            folderpath=pathToOutputDir,
            title=False,
            append="_original",
            readability=False
        ).get()

    def test_printpdf_original_rename_function(self):

        from polyglot import printpdf
        pdf = printpdf(
            log=log,
            settings=settings,
            url="https://en.wikipedia.org/wiki/Volkswagen",
            folderpath=pathToOutputDir,
            title="Cars",
            append="_original",
            readability=False
        ).get()

        # x-print-testpage-for-pessto-marshall-web-object

    # x-class-to-test-named-worker-function



#!/usr/local/bin/python
# encoding: utf-8
"""
*using the Readability API to clean up a local html file*

:Author:
    David Young

:Date Created:
    October  1, 2016
"""
################# GLOBAL IMPORTS ####################
import sys
import os
import codecs
import re
os.environ['TERM'] = 'vt100'
from fundamentals import tools
from polyglot import authenticate


class htmlCleaner():
    """
    *A parser/cleaner to strip a webpage article of all cruft and neatly present it with some nice css*

    **Key Arguments:**
        - ``log`` -- logger
        - ``settings`` -- the settings dictionary
        - ``url`` -- the URL to the HTML page to parse and clean
        - ``outputDirectory`` -- path to the directory to save the output html file to
        - ``title`` -- title of the document to save. If *False* will take the title of the HTML page as the filename. Default *False*.
        - ``style`` -- add polyglot's styling to the HTML document. Default *True*
        - ``metadata`` -- include metadata in generated HTML. Default *True*
        - ``h1`` -- include title as H1 at the top of the doc. Default *True*

    **Usage:**

        To generate the HTML page, using the title of the webpage as the filename:

        .. code-block:: python 

            from polyglot import htmlCleaner
            cleaner = htmlCleaner(
                log=log,
                settings=settings,
                url="http://www.thespacedoctor.co.uk/blog/2016/09/26/mysqlSucker-index.html",
                outputDirectory="/tmp"
            )
            cleaner.clean()  

        Or specify the title of the document and remove styling, metadata and title:

        .. code-block:: python 

            from polyglot import htmlCleaner
            cleaner = htmlCleaner(
                log=log,
                settings=settings,
                url="http://www.thespacedoctor.co.uk/blog/2016/09/26/mysqlSucker-index.html",
                outputDirectory="/tmp",
                title="my_clean_doc",
                style=False,
                metadata=False,
                h1=False
            )
            cleaner.clean() 

    """
    # INITIALISATION

    def __init__(
            self,
            log,
            settings,
            url,
            outputDirectory=False,
            title=False,
            style=True,
            metadata=True,
            h1=True
    ):
        self.log = log
        log.debug("instansiating a new 'htmlCleaner' object")
        self.settings = settings
        self.url = url
        self.outputDirectory = outputDirectory
        self.title = title
        self.style = style
        self.metadata = metadata
        self.h1 = h1

        # INITIAL ACTIONS
        # AUTHENTICATE AGAINST READABILITY WEBAPP PARSER CLIENT
        self.parser_client = authenticate.authenticate(
            log=self.log,
            settings=self.settings
        ).get()

        return None

    def clean(
            self):
        """*parse and clean the html document with readability parser*

        **Return:**
            - ``filePath`` -- path to the cleaned HTML document

        **Usage:**

            See class usage 
        """
        self.log.info('starting the ``clean`` method')

        url = self.url

        # PARSE THE CONTENT OF THE WEBPAGE AT THE URL
        parser_response = self.parser_client.get_article(self.url)
        if "503" in str(parser_response):
            return None
        article = parser_response.json()

        # GRAB THE CSS USED TO STYLE THE WEBPAGE/PDF CONTENT
        if self.style:
            moduleDirectory = os.path.dirname(__file__)
            cssFile = moduleDirectory + "/css/main.css"
            pathToReadFile = cssFile
            readFile = codecs.open(pathToReadFile, encoding='utf-8', mode='r')
            thisCss = readFile.read()
            readFile.close()
        else:
            thisCss = ""

        # CATCH ERRORS
        if "error" in article and article["error"] == True:
            print url
            print "    " + article["messages"]
            return None
        try:
            text = article["content"]
        except:
            print "Can't decode the text of %(url)s - moving on" % locals()
            return None

        # COMMON FIXES TO HTML TO RENDER CORRECTLY
        regex = re.compile(
            u'<span class="mw-editsection"><span class="mw-editsection-bracket">.*"mw-editsection-bracket">]')
        text = regex.sub(u"", text)
        regex2 = re.compile(
            u'\<sup class="noprint.*better source needed\<\/span\>\<\/a\>\<\/i\>\]\<\/sup\>', re.I)
        text = regex2.sub(u"", text)
        regex2 = re.compile(
            u'\<a href="https\:\/\/en\.wikipedia\.org\/wiki\/.*(\#.*)"\>\<span class=\"tocnumber\"\>', re.I)
        text = regex2.sub(u'<a href="\g<1>"><span class="tocnumber">', text)

        # GRAB HTML TITLE IF NOT SET IN ARGUMENTS
        if self.title == False:
            title = article["title"].encode("utf-8", "ignore")
            title = title.decode("utf-8")
            title = title.encode("ascii", "ignore")
            rstrings = """:/"&\\'"""
            for i in rstrings:
                title = title.replace(i, "")

            # USE DATETIME IF TITLE STILL NOT SET
            if len(title) == 0:
                from datetime import datetime, date, time
                now = datetime.now()
                title = now.strftime("%Y%m%dt%H%M%S")
            self.title = title

        title = self.title.replace(".html", "")
        pageTitle = title.replace("_", " ")

        # REGENERATE THE HTML DOCUMENT WITH CUSTOM STYLE
        filePath = self.outputDirectory + "/" + title + ".html"
        writeFile = codecs.open(
            filePath, encoding='utf-8', mode='w')
        if self.metadata:
            metadata = "<title>%(title)s</title>" % locals()
        else:
            metadata = ""

        if self.h1:
            h1 = "<h1>%(pageTitle)s</h1>" % locals()
        else:
            h1 = ""
        content = u"""
<!DOCTYPE html>
<html>
<head>
<meta charset="utf-8">
%(metadata)s 

<style>
%(thisCss)s
</style>

</head>
<body>

%(h1)s 
<a href="%(url)s">original source</a>
</br></br>


%(text)s 
</body>
</html>""" % locals()
        writeFile.write(content)
        writeFile.close()

        self.log.info('completed the ``clean`` method')
        return filePath

    # xt-class-method



#!/usr/local/bin/python
# encoding: utf-8
"""
*Convert various document formats to epub or mobi*

:Author:
    David Young

:Date Created:
    October  9, 2016
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
from fundamentals import tools
import codecs
from subprocess import Popen, PIPE
from datetime import datetime, date, time


class ebook():
    """
    *The worker class for the ebook module*

    **Key Arguments:**
        - ``log`` -- logger
        - ``settings`` -- the settings dictionary
        - ``urlOrPath`` -- the url or path to the content source
        - ``bookFormat`` -- the output format (epub, mobi)
        - ``outputDirectory`` -- path to the directory to save the output html file to. 
        - ``title`` -- the title of the output document. I. False then use the title of the original source. Default *False*
        - ``header`` -- content to add before the article/book content in the resulting ebook. Default *False*
        - ``footer`` -- content to add at the end of the article/book content in the resulting ebook. Default *False*

    **Usage:**
        .. todo::

            - add usage info
            - create a sublime snippet for usage
            - update the package tutorial if needed

        .. code-block:: python 

            usage code   
    """
    # Initialisation

    def __init__(
            self,
            log,
            settings,
            urlOrPath,
            outputDirectory,
            bookFormat,
            title=False,
            header=False,
            footer=False
    ):
        self.log = log
        log.debug("instansiating a new 'ebook' object")
        self.settings = settings
        self.title = title
        self.header = header
        self.footer = footer
        self.urlOrPath = urlOrPath
        self.outputDirectory = outputDirectory
        self.format = bookFormat
        # xt-self-arg-tmpx

        # Initial Actions

        return None

    def get(self):
        """
        *get the ebook object*

        **Return:**
            - ``ebook``

        **Usage:**
        .. todo::

            - add usage info
            - create a sublime snippet for usage
            - update the package tutorial if needed

        To generate an ebook from an article found on the web, using the webpages's title as the filename for the book:

        .. code-block:: python 

            from polyglot import ebook
            epub = ebook(
                log=log,
                settings=settings,
                urlOrPath="http://www.thespacedoctor.co.uk/blog/2016/09/26/mysqlSucker-index.html",
                title=False,
                bookFormat="epub",
                outputDirectory="/path/to/output/folder"
            )
            pathToEpub = epub.get()

        To add a header and footer to the epub book, and specify the title/filename for the book:

        .. code-block:: python 

            from polyglot import ebook
            epub = ebook(
                log=log,
                settings=settings,
                urlOrPath="http://www.thespacedoctor.co.uk/blog/2016/09/26/mysqlSucker-index.html",
                title="MySQL Sucker",
                bookFormat="epub",
                outputDirectory="/path/to/output/folder",
                header='<a href="http://www.thespacedoctor.co.uk">thespacedoctor</a>',
                footer='<a href="http://www.thespacedoctor.co.uk">thespacedoctor</a>'
            )
            pathToEpub = epub.get()
        """
        self.log.info('starting the ``get`` method')

        if self.format == "epub":
            if self.urlOrPath[:4] == "http" or self.urlOrPath[:4] == "www.":
                ebook = self._url_to_epub()

        if self.format == "mobi":
            if self.urlOrPath[:4] == "http" or self.urlOrPath[:4] == "www.":
                epub = self._url_to_epub()
            ebook = self._epub_to_mobi(
                epubPath=epub,
                deleteEpub=False
            )

        self.log.info('completed the ``get`` method')
        return ebook

    def _url_to_epub(
            self):
        """*generate the epub book from a URL*
        """
        self.log.info('starting the ``_url_to_epub`` method')

        from polyglot import htmlCleaner
        cleaner = htmlCleaner(
            log=self.log,
            settings=self.settings,
            url=self.urlOrPath,
            outputDirectory=self.outputDirectory,
            title=self.title,  # SET TO FALSE TO USE WEBPAGE TITLE,
            style=False,  # add simpdf's styling to the HTML document
            metadata=True,  # include metadata in generated HTML (e.g. title),
            h1=False  # include title as H1 at the top of the doc
        )
        html = cleaner.clean()

        if self.footer:
            footer = self._tmp_html_file(self.footer)
            footer = '"%(footer)s"' % locals()
        else:
            footer = ""

        if self.header:
            header = self._tmp_html_file(self.header)
            header = '"%(header)s"' % locals()
        else:
            header = ""

        # HTML SOURCE FILE
        epub = html.replace(".html", ".epub")
        pandoc = self.settings["executables"]["pandoc"]

        cmd = """%(pandoc)s -S -s -f html -t epub3 %(header)s "%(html)s" %(footer)s -o "%(epub)s" """ % locals(
        )
        p = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
        stdout, stderr = p.communicate()
        self.log.debug('output: %(stdout)s' % locals())

        try:
            with open(epub):
                pass
            fileExists = True
        except IOError:
            fileExists = False
            raise IOError(
                "the epub %s does not exist on this machine, here is the failure message: %s" % (epub, stderr))

        os.remove(html)

        self.log.info('completed the ``_url_to_epub`` method')
        return epub

    def _tmp_html_file(
            self,
            content):
        """*create a tmp html file with some content used for the header or footer of the ebook*

        **Key Arguments:**
            - ``content`` -- the content to include in the HTML file.
        """
        self.log.info('starting the ``_tmp_html_file`` method')

        content = """

<hr>
<div style="text-align: center">
%(content)s 
</div>
<hr>

""" % locals()

        now = datetime.now()
        now = now.strftime("%Y%m%dt%H%M%S%f")
        pathToWriteFile = "/tmp/%(now)s.html" % locals()
        try:
            self.log.debug("attempting to open the file %s" %
                           (pathToWriteFile,))
            writeFile = codecs.open(
                pathToWriteFile, encoding='utf-8', mode='w')
        except IOError, e:
            message = 'could not open the file %s' % (pathToWriteFile,)
            self.log.critical(message)
            raise IOError(message)
        writeFile.write(content)
        writeFile.close()

        self.log.info('completed the ``_tmp_html_file`` method')
        return pathToWriteFile

    def _epub_to_mobi(
            self,
            epubPath,
            deleteEpub=False):
        """*convert the give epub to mobi format using kindlegen*

        **Key Arguments:**
            - ``epubPath`` -- path to the epub book
            - ``deleteEpub`` -- delete the epub when mobi is generated. Default *False*

        **Return:**
            - ``mobi`` -- the path to the generated mobi book
        """
        self.log.info('starting the ``_epub_to_mobi`` method')

        mobi = epubPath.replace(".epub", ".mobi")
        kindlegen = self.settings["executables"]["kindlegen"]
        cmd = """%(kindlegen)s "%(epubPath)s" """ % locals(
        )
        p = Popen(cmd, stdout=PIPE, stderr=PIPE, shell=True)
        stdout, stderr = p.communicate()
        self.log.debug('output: %(stdout)s' % locals())

        try:
            with open(mobi):
                pass
            fileExists = True
        except IOError:
            fileExists = False
            raise IOError(
                "the mobi %s does not exist on this machine" % (epubPath,))

        if deleteEpub:
            os.remove(epubPath)

        self.log.info('completed the ``_epub_to_mobi`` method')
        return mobi



#!/usr/local/bin/python
# encoding: utf-8
"""
*Authenticate against readability api*

:Author:
    David Young

:Date Created:
    September 28, 2015
"""
################# GLOBAL IMPORTS ####################
import sys
import os
os.environ['TERM'] = 'vt100'
import readline
import glob
import pickle
from docopt import docopt
from fundamentals import tools, times


class authenticate():

    """
    *Authenticate against readability api*

    The parser api token is to be found in the polyglot settings file (``~/.config/polyglot/polyglot.yaml``). 

    Read more about readability parser `here <https://www.readability.com/developers/api>`_, or to get your readability parser key sign up to readability and `grab the key here <https://www.readability.com/settings/account>`_

    **Key Arguments:**
        - ``log`` -- logger
        - ``settings`` -- the settings dictionary

    **Usage:**

        To setup a readability parser client:

        .. code-block:: python 

            from polyglot import authenticate
            parserClient = authenticate(
                log=log,
                settings=settings
            ).get() 
    """

    def __init__(
            self,
            log,
            settings=False,

    ):
        self.log = log
        log.debug("instansiating a new 'authenticate' object")
        self.settings = settings
        # xt-self-arg-tmpx

        # Initial Actions

        return None

    def get(self):
        """
        *Get the readability parser client*

        **Return:**
            - ``parserClient`` -- the readability parser client
        """
        self.log.info('starting the ``get`` method')

        from readability import ParserClient
        os.environ['READABILITY_PARSER_TOKEN'] = self.settings[
            "readability"]["parser api token"]

        parser_client = ParserClient()

        self.log.info('completed the ``get`` method')
        return parser_client



import os
import nose
import shutil
import yaml
from polyglot.utKit import utKit

from fundamentals import tools

su = tools(
    arguments={"settingsFile": None},
    docString=__doc__,
    logLevel="DEBUG",
    options_first=False,
    projectName="polyglot"
)
arguments, settings, log, dbConn = su.setup()


# SETUP AND TEARDOWN FIXTURE FUNCTIONS FOR THE ENTIRE MODULE
moduleDirectory = os.path.dirname(__file__)
utKit = utKit(moduleDirectory)
log, dbConn, pathToInputDir, pathToOutputDir = utKit.setupModule()
utKit.tearDownModule()

# load settings
stream = file(
    pathToInputDir + "/example_settings.yaml", 'r')
settings = yaml.load(stream)
stream.close()

import shutil
try:
    shutil.rmtree(pathToOutputDir)
except:
    pass

# Recursively create missing directories
if not os.path.exists(pathToOutputDir):
    os.makedirs(pathToOutputDir)


class test_authenticate():

    def test_authenticate_function(self):

        from polyglot import authenticate
        parserClient = authenticate(
            log=log,
            settings=settings
        ).get()

        # x-print-testpage-for-pessto-marshall-web-object

    # x-class-to-test-named-worker-function
